@@ -61,6 +61,8 @@ class Formatter(BaseFormatter):  # type: ignore
         result = f"[{match.level}][bold][link={match.rule.url}]{self.escape(match.tag)}[/link][/][/][dim]:[/] [{match.level}]{self.escape(match.message)}[/]"
         if match.level != "error":
             result += f" [dim][{match.level}]({match.level})[/][/]"
+        if match.ignored:
+            result += " [dim]# ignored[/]"
         result += (
             "\n"
             f"[filename]{self._format_path(match.filename or '')}[/]:{match.position}"
@@ -151,19 +153,20 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
             if match.rule.url:
                 # https://github.com/codeclimate/platform/issues/68
                 issue["url"] = match.rule.url
-            issue["severity"] = self._severity_to_level(match)
+            issue["severity"] = self._remap_severity(match)
             issue["description"] = self.escape(str(match.message))
             issue["fingerprint"] = hashlib.sha256(
                 repr(match).encode("utf-8")
             ).hexdigest()
             issue["location"] = {}
             issue["location"]["path"] = self._format_path(match.filename or "")
-            issue["location"]["lines"] = {}
             if match.column:
-                issue["location"]["lines"]["begin"] = {}
-                issue["location"]["lines"]["begin"]["line"] = match.linenumber
-                issue["location"]["lines"]["begin"]["column"] = match.column
+                issue["location"]["positions"] = {}
+                issue["location"]["positions"]["begin"] = {}
+                issue["location"]["positions"]["begin"]["line"] = match.linenumber
+                issue["location"]["positions"]["begin"]["column"] = match.column
             else:
+                issue["location"]["lines"] = {}
                 issue["location"]["lines"]["begin"] = match.linenumber
             if match.details:
                 issue["content"] = {}
@@ -171,24 +174,19 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
             # Append issue to result list
             result.append(issue)
 
-        return json.dumps(result)
+        return json.dumps(result, sort_keys=False, indent=2)
 
     @staticmethod
-    def _severity_to_level(match: MatchError) -> str:
-        if match.level != "error":
-            return "info"
-        severity = match.rule.severity
-
-        if severity in ["LOW"]:
+    def _remap_severity(match: MatchError) -> str:
+        # level is not part of CodeClimate specification, but there is
+        # no other way to expose that info. We recommend switching to
+        # SARIF format which is better suited for interoperability.
+        #
+        # Out current implementation will return `major` for all errors and
+        # `warning` for all warnings. We may revisit this in the future.
+        if match.level == "warning":
             return "minor"
-        if severity in ["MEDIUM"]:
-            return "major"
-        if severity in ["HIGH"]:
-            return "critical"
-        if severity in ["VERY_HIGH"]:
-            return "blocker"
-        # VERY_LOW, INFO or anything else
-        return "info"
+        return "major"
 
 
 class SarifFormatter(BaseFormatter[Any]):
@@ -243,9 +241,7 @@ class SarifFormatter(BaseFormatter[Any]):
             "runs": runs,
         }
 
-        return json.dumps(
-            report, default=lambda o: o.__dict__, sort_keys=False, indent=2
-        )
+        return json.dumps(report, sort_keys=False, indent=2)
 
     def _extract_results(
         self, matches: list[MatchError]
@@ -253,17 +249,17 @@ class SarifFormatter(BaseFormatter[Any]):
         rules = {}
         results = []
         for match in matches:
-            if match.rule.id not in rules:
-                rules[match.rule.id] = self._to_sarif_rule(match)
+            if match.tag not in rules:
+                rules[match.tag] = self._to_sarif_rule(match)
             results.append(self._to_sarif_result(match))
         return list(rules.values()), results
 
     def _to_sarif_rule(self, match: MatchError) -> dict[str, Any]:
         rule: dict[str, Any] = {
-            "id": match.rule.id,
+            "id": match.tag,
             "name": match.tag,
             "shortDescription": {
-                "text": self.escape(str(match.message)),
+                "text": str(match.message),
             },
             "defaultConfiguration": {
                 "level": self._to_sarif_level(match),
@@ -274,15 +270,15 @@ class SarifFormatter(BaseFormatter[Any]):
             "helpUri": match.rule.url,
             "properties": {"tags": match.rule.tags},
         }
-        if match.rule.link:
-            rule["helpUri"] = match.rule.link
         return rule
 
     def _to_sarif_result(self, match: MatchError) -> dict[str, Any]:
         result: dict[str, Any] = {
-            "ruleId": match.rule.id,
+            "ruleId": match.tag,
             "message": {
-                "text": match.details,
+                "text": str(match.details)
+                if str(match.details)
+                else str(match.message),
             },
             "locations": [
                 {
