@@ -8,10 +8,11 @@ import re
 import sys
 from argparse import Namespace
 from collections import defaultdict
+from collections.abc import Iterable, Iterator, MutableMapping, MutableSequence
 from functools import lru_cache
 from importlib import import_module
 from pathlib import Path
-from typing import Any, Iterable, Iterator, MutableMapping, MutableSequence, cast
+from typing import Any, cast
 
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
@@ -154,23 +155,24 @@ class AnsibleLintRule(BaseRule):
         ):
             return matches
 
-        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file)
-        for raw_task, task, skipped_tags, error in tasks_iterator:
-            if error is not None:
+        for task in ansiblelint.utils.task_in_list(
+            data=file.data, kind=file.kind, filename=str(file.path)
+        ):
+            if task.error is not None:
                 # normalize_task converts AnsibleParserError to MatchError
-                return [error]
+                return [task.error]
 
             if (
-                self.id in skipped_tags
-                or ("action" not in task)
-                or "skip_ansible_lint" in task.get("tags", [])
+                self.id in task.skip_tags
+                or ("action" not in task.normalized_task)
+                or "skip_ansible_lint" in task.normalized_task.get("tags", [])
             ):
                 continue
 
             if self.needs_raw_task:
-                task["__raw_task__"] = raw_task
+                task.normalized_task["__raw_task__"] = task.raw_task
 
-            result = self.matchtask(task, file=file)
+            result = self.matchtask(task.normalized_task, file=file)
             if not result:
                 continue
 
@@ -180,13 +182,15 @@ class AnsibleLintRule(BaseRule):
                 # https://github.com/PyCQA/pylint/issues/6044
                 # pylint: disable=not-an-iterable
                 for match in result:
-                    if match.tag in skipped_tags:
+                    if match.tag in task.skip_tags:
                         continue
-                    self._enrich_matcherror_with_task_details(match, task)
+                    self._enrich_matcherror_with_task_details(
+                        match, task.normalized_task
+                    )
                     matches.append(match)
                 continue
             if isinstance(result, MatchError):
-                if result.tag in skipped_tags:
+                if result.tag in task.skip_tags:
                     continue
                 match = result
             else:  # bool or string
@@ -195,11 +199,11 @@ class AnsibleLintRule(BaseRule):
                     message = result
                 match = self.create_matcherror(
                     message=message,
-                    linenumber=task[LINE_NUMBER_KEY],
+                    linenumber=task.normalized_task[LINE_NUMBER_KEY],
                     filename=file,
                 )
 
-            self._enrich_matcherror_with_task_details(match, task)
+            self._enrich_matcherror_with_task_details(match, task.normalized_task)
             matches.append(match)
         return matches
 
@@ -336,11 +340,7 @@ def load_plugins(  # noqa: max-complexity: 11
         # load all modules in the directory
         for f in Path(directory).glob("*.py"):
             if "__" not in f.stem and f.stem not in "conftest":
-                try:
-                    import_module(f"{f.stem}")
-
-                except ImportError as exc:
-                    _logger.warning("Ignore loading rule from %s due to %s", f, exc)
+                import_module(f"{f.stem}")
     # restore sys.path
     sys.path = orig_sys_path
 
@@ -393,8 +393,9 @@ class RulesCollection:
             self.register(rule, conditional=conditional)
         self.rules = sorted(self.rules)
 
-        # when we have a profile we unload some of the rules
-        if profile_name:
+        # When we have a profile we unload some of the rules
+        # But we do include all rules when listing all rules or tags
+        if profile_name and not (self.options.list_rules or self.options.list_tags):
             filter_rules_with_profile(self.rules, profile_name)
 
     def register(self, obj: AnsibleLintRule, conditional: bool = False) -> None:
