@@ -6,18 +6,10 @@ import functools
 import logging
 import os
 import re
+from collections.abc import Iterator, Sequence
 from io import StringIO
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    Iterator,
-    Pattern,
-    Tuple,
-    Union,
-    cast,
-)
+from re import Pattern
+from typing import TYPE_CHECKING, Any, Callable, Union, cast
 
 import ruamel.yaml.events
 from ruamel.yaml.comments import CommentedMap, CommentedSeq, Format
@@ -33,10 +25,13 @@ from ruamel.yaml.scalarint import ScalarInt
 from ruamel.yaml.tokens import CommentToken
 from yamllint.config import YamlLintConfig
 
-from ansiblelint.constants import NESTED_TASK_KEYS, PLAYBOOK_TASK_KEYWORDS
-from ansiblelint.errors import MatchError
+from ansiblelint.constants import (
+    ANNOTATION_KEYS,
+    NESTED_TASK_KEYS,
+    PLAYBOOK_TASK_KEYWORDS,
+    SKIPPED_RULES_KEY,
+)
 from ansiblelint.file_utils import Lintable
-from ansiblelint.utils import get_action_tasks, normalize_task
 
 if TYPE_CHECKING:
     # noinspection PyProtectedMember
@@ -62,9 +57,27 @@ rules:
   braces:
     min-spaces-inside: 0  # yamllint defaults to 0
     max-spaces-inside: 1  # yamllint defaults to 0
+  octal-values:
+    forbid-implicit-octal: true  # yamllint defaults to false
+    forbid-explicit-octal: true  # yamllint defaults to false
 """
 
 
+def deannotate(data: Any) -> Any:
+    """Remove our annotations like __file__ and __line__ and return a JSON serializable object."""
+    if isinstance(data, dict):
+        result = data.copy()
+        for key, value in data.items():
+            if key in ANNOTATION_KEYS:
+                del result[key]
+            else:
+                result[key] = deannotate(value)
+        return result
+    if isinstance(data, list):
+        return [deannotate(item) for item in data if item not in ANNOTATION_KEYS]
+    return data
+
+
 @functools.lru_cache(maxsize=1)
 def load_yamllint_config() -> YamlLintConfig:
     """Load our default yamllint config and any customized override file."""
@@ -93,60 +106,9 @@ def load_yamllint_config() -> YamlLintConfig:
     return config
 
 
-def iter_tasks_in_file(
-    lintable: Lintable,
-) -> Iterator[tuple[dict[str, Any], dict[str, Any], list[str], MatchError | None]]:
-    """Iterate over tasks in file.
-
-    This yields a 4-tuple of raw_task, normalized_task, skip_tags, and error.
-
-    raw_task:
-        When looping through the tasks in the file, each "raw_task" is minimally
-        processed to include these special keys: __line__, __file__, skipped_rules.
-    normalized_task:
-        When each raw_task is "normalized", action shorthand (strings) get parsed
-        by ansible into python objects and the action key gets normalized. If the task
-        should be skipped (skipped is True) or normalizing it fails (error is not None)
-        then this is just the raw_task instead of a normalized copy.
-    skip_tags:
-        List of tags found to be skipped, from tags block or noqa comments
-    error:
-        This is normally None. It will be a MatchError when the raw_task cannot be
-        normalized due to an AnsibleParserError.
-
-    :param lintable: The playbook or tasks/handlers yaml file to get tasks from
-
-    Yields raw_task, normalized_task, skipped, error
-    """
-    data = lintable.data
-    if not data:
-        return
-
-    raw_tasks = get_action_tasks(data, lintable)
-
-    for raw_task in raw_tasks:
-        err: MatchError | None = None
-
-        skip_tags: list[str] = raw_task.get("skipped_rules", [])
-
-        try:
-            normalized_task = normalize_task(raw_task, str(lintable.path))
-        except MatchError as err:
-            # normalize_task converts AnsibleParserError to MatchError
-            yield raw_task, raw_task, skip_tags, err
-            return
-
-        if "skip_ansible_lint" in (raw_task.get("tags") or []):
-            skip_tags.append("skip_ansible_lint")
-        if skip_tags:
-            yield raw_task, normalized_task, skip_tags, err
-            continue
-
-        yield raw_task, normalized_task, skip_tags, err
-
-
 def nested_items_path(
     data_collection: dict[Any, Any] | list[Any],
+    ignored_keys: Sequence[str] = (),
 ) -> Iterator[tuple[Any, Any, list[str | int]]]:
     """Iterate a nested data structure, yielding key/index, value, and parent_path.
 
@@ -207,12 +169,15 @@ def nested_items_path(
     # valid data, we better ignore NoneType
     if data_collection is None:
         return
-    yield from _nested_items_path(data_collection=data_collection, parent_path=[])
+    yield from _nested_items_path(
+        data_collection=data_collection, parent_path=[], ignored_keys=ignored_keys
+    )
 
 
 def _nested_items_path(
     data_collection: dict[Any, Any] | list[Any],
     parent_path: list[str | int],
+    ignored_keys: Sequence[str] = (),
 ) -> Iterator[tuple[Any, Any, list[str | int]]]:
     """Iterate through data_collection (internal implementation of nested_items_path).
 
@@ -222,7 +187,7 @@ def _nested_items_path(
     """
     # we have to cast each convert_to_tuples assignment or mypy complains
     # that both assignments (for dict and list) do not have the same type
-    convert_to_tuples_type = Callable[[], Iterator[Tuple[Union[str, int], Any]]]
+    convert_to_tuples_type = Callable[[], Iterator[tuple[Union[str, int], Any]]]
     if isinstance(data_collection, dict):
         convert_data_collection_to_tuples = cast(
             convert_to_tuples_type, functools.partial(data_collection.items)
@@ -237,7 +202,7 @@ def _nested_items_path(
             f"of type '{type(data_collection)}'"
         )
     for key, value in convert_data_collection_to_tuples():
-        if key in ("skipped_rules", "__file__", "__line__"):
+        if key in (SKIPPED_RULES_KEY, "__file__", "__line__", *ignored_keys):
             continue
         yield key, value, parent_path
         if isinstance(value, (dict, list)):
@@ -563,8 +528,8 @@ class FormattedEmitter(Emitter):
     #     https://github.com/python/mypy/issues/4125
     #     To silence we have to ignore[override] both the @property and the method.
 
-    @property  # type: ignore[override]
-    def best_sequence_indent(self) -> int:  # type: ignore[override]
+    @property
+    def best_sequence_indent(self) -> int:
         """Return the configured sequence_indent or 2 for root level."""
         return 2 if self._is_root_level_sequence else self._sequence_indent
 
@@ -573,8 +538,8 @@ class FormattedEmitter(Emitter):
         """Configure how many columns to indent each sequence item (including the '-')."""
         self._sequence_indent = value
 
-    @property  # type: ignore[override]
-    def sequence_dash_offset(self) -> int:  # type: ignore[override]
+    @property
+    def sequence_dash_offset(self) -> int:
         """Return the configured sequence_dash_offset or 0 for root level."""
         return 0 if self._is_root_level_sequence else self._sequence_dash_offset
 
@@ -586,6 +551,16 @@ class FormattedEmitter(Emitter):
     def choose_scalar_style(self) -> Any:
         """Select how to quote scalars if needed."""
         style = super().choose_scalar_style()
+        if (
+            style == ""
+            and self.event.value.startswith("0")
+            and len(self.event.value) > 1
+        ):
+            if self.event.tag == "tag:yaml.org,2002:int" and self.event.implicit[0]:
+                # ensures that "0123" string does not lose its quoting
+                self.event.tag = "tag:yaml.org,2002:str"
+                self.event.implicit = (True, True, True)
+            return '"'
         if style != "'":
             # block scalar, double quoted, etc.
             return style
@@ -674,7 +649,7 @@ class FormattedEmitter(Emitter):
         """Clean up extra new lines and spaces in comments.
 
         ruamel.yaml treats new or empty lines as comments.
-        See: https://stackoverflow.com/a/42712747/1134951
+        See: https://stackoverflow.com/questions/42708668/removing-all-blank-lines-but-not-comments-in-ruamel-yaml/42712747#42712747
         """
         value: str = comment.value
         if (
@@ -891,10 +866,10 @@ class FormattedYAML(YAML):
                 elif quote_type == "double":
                     config["preferred_quote"] = '"'
 
-        return cast(Dict[str, Union[bool, int, str]], config)
+        return cast(dict[str, Union[bool, int, str]], config)
 
     @property  # type: ignore[override]
-    def version(self) -> str | tuple[int, int]:  # type: ignore[override]
+    def version(self) -> str | tuple[int, int]:
         """Return the YAML version used to parse or dump.
 
         Ansible uses PyYAML which only supports YAML 1.1. ruamel.yaml defaults to 1.2.
@@ -988,7 +963,7 @@ class FormattedYAML(YAML):
         Preserve blank lines despite extra whitespace.
         Preserve any preamble (aka header) comments before "---".
 
-        For more on preamble comments, see: https://stackoverflow.com/a/70287507/1134951
+        For more on preamble comments, see: https://stackoverflow.com/questions/70286108/python-ruamel-yaml-package-how-to-get-header-comment-lines/70287507#70287507
         """
         text = self._whitespace_only_lines_re.sub("", text)
 
@@ -1030,7 +1005,7 @@ class FormattedYAML(YAML):
         Make sure there's only one newline at the end of the file.
 
         Fix the indent of full-line comments to match the indent of the next line.
-        See: https://stackoverflow.com/a/71355688/1134951
+        See: https://stackoverflow.com/questions/71354698/how-can-i-use-the-ruamel-yaml-rtsc-mode/71355688#71355688
         Also, removes "#" protection from strings that prevents them from being
         identified as full line comments in post-processing.
 
@@ -1076,3 +1051,33 @@ class FormattedYAML(YAML):
             FormattedEmitter.drop_octothorpe_protection(line) for line in lines
         )
         return text
+
+
+def clean_json(
+    obj: Any,
+    func: Callable[[str], Any] = lambda key: key.startswith("__")
+    if isinstance(key, str)
+    else False,
+) -> Any:
+    """
+    Remove all keys matching the condition from a nested JSON-like object.
+
+    :param obj: a JSON like object to clean, also returned for chaining.
+    :param func: a callable that takes a key in argument and return True for each key to delete
+    """
+    if isinstance(obj, dict):
+        for key in list(obj.keys()):
+            if func(key):
+                del obj[key]
+            else:
+                clean_json(obj[key], func)
+    elif isinstance(obj, list):
+        for i in reversed(range(len(obj))):
+            if func(obj[i]):
+                del obj[i]
+            else:
+                clean_json(obj[i], func)
+    else:
+        # neither a dict nor a list, do nothing
+        pass
+    return obj
